@@ -1,5 +1,6 @@
 import dataclasses
 import time
+from concurrent.futures import ThreadPoolExecutor, Future
 from threading import RLock, Thread, Condition
 from typing import Optional, Type, Callable, Any, List, Dict, Tuple, Union
 from inspect import signature
@@ -81,7 +82,8 @@ class ConsumerLoop:
                  reconnection_retries: int = 50,
                  execution_delay: float = 5,
                  prefetch_count: int = 3,
-                 global_prefetch: bool = False):
+                 global_prefetch: bool = False,
+                 dispatch_sleep_seconds: float = 1.0):
         """
             Class that manages a channel, implements reconnecting mechanism and allow listeners for queues.
             It's recommended to have only one listener per loop due to Pika limitations
@@ -92,6 +94,7 @@ class ConsumerLoop:
             :param execution_delay: seconds to wait after initialization before start consuming
             :param prefetch_count: number of messages to be prefetched in advanced
             :param global_prefetch: set prefetch limit on connection instead of channel
+            :param dispatch_sleep_seconds: period of seconds to sleep while waiting for the response of a dispatched message
         """
         assert connection is not None
 
@@ -108,6 +111,9 @@ class ConsumerLoop:
         self._global_prefetch: bool = global_prefetch
         self._loop: Thread = Thread(name='consumer-loop', target=ConsumerLoop._consumer_loop, args=[self], daemon=True)
         self._add_listener_condition: Condition = Condition()
+        self._thread_pool: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=prefetch_count,
+                                                                   thread_name_prefix='queue-message-dispatcher-')
+        self._dispatch_sleep_seconds: float = dispatch_sleep_seconds
 
         self._open_channel()
         self._loop.start()
@@ -178,7 +184,15 @@ class ConsumerLoop:
             unconsumable: bool = False
 
             try:
-                ack, response = listener.on_message(message)
+                future: Future = self._thread_pool.submit(listener.on_message, message)
+
+                while future.running():
+                    channel.connection.sleep(self._dispatch_sleep_seconds)  # keeping the connection alive
+
+                if future.exception() is not None:
+                    raise future.exception()
+                else:
+                    ack, response = future.result()
 
                 if properties.reply_to is not None:
                     try:
